/*******************************************************************************
!   Copyright(C) 2011-2012 Intel Corporation. All Rights Reserved.
!
!   The source code, information  and  material ("Material") contained herein is
!   owned  by Intel Corporation or its suppliers or licensors, and title to such
!   Material remains  with Intel Corporation  or its suppliers or licensors. The
!   Material  contains proprietary information  of  Intel or  its  suppliers and
!   licensors. The  Material is protected by worldwide copyright laws and treaty
!   provisions. No  part  of  the  Material  may  be  used,  copied, reproduced,
!   modified, published, uploaded, posted, transmitted, distributed or disclosed
!   in any way  without Intel's  prior  express written  permission. No  license
!   under  any patent, copyright  or  other intellectual property rights  in the
!   Material  is  granted  to  or  conferred  upon  you,  either  expressly,  by
!   implication, inducement,  estoppel or  otherwise.  Any  license  under  such
!   intellectual  property  rights must  be express  and  approved  by  Intel in
!   writing.
!
!   *Third Party trademarks are the property of their respective owners.
!
!   Unless otherwise  agreed  by Intel  in writing, you may not remove  or alter
!   this  notice or  any other notice embedded  in Materials by Intel or Intel's
!   suppliers or licensors in any way.
!
!*******************************************************************************
!   Content:
!       Utility functions for SGEMM, DGEMM, CGEMM, ZGEMM benchmarks
!******************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <immintrin.h>

#include "utils.h"

#if defined(__MIC__) && !defined(__INTEL_OFFLOAD)

/* Memory allocation for native execution on an Intel(R) Xeon Phi(TM)
 * coprocessor. Requires allocation of huge/2M pages to be enabled (this is
 * the default setting). */

#include <sys/mman.h>
#include <linux/mman.h>

#define HUGE_PAGE_SIZE (2 * 1024 * 1024)
#define RNDUP(val, bord) ((((val) + bord - 1) / bord) * bord)

void *bench_malloc(size_t size)
{
	size_t size_aligned = RNDUP(size, HUGE_PAGE_SIZE);
	void *ptr = mmap(NULL, size_aligned, PROT_READ | PROT_WRITE,
			MAP_PRIVATE | MAP_HUGETLB | MAP_ANONYMOUS | MAP_POPULATE, -1, 0);
	if (ptr != MAP_FAILED)
		return ptr;

	size_t mbytes = size_aligned / (1024 * 1024);
	size_t npages = size_aligned / HUGE_PAGE_SIZE;
	printf("Error: Could not allocate memory (%zu MBytes) with 2M pages.\n"
		"       To enable allocation of memory with 2M pages execute\n"
		"       'echo %zu > /proc/sys/vm/nr_overcommit_hugepages'\n"
		"       as root\n", mbytes, npages);
	exit(1);
}

void bench_free(void *ptr, size_t size)
{
	munmap(ptr, RNDUP(size, HUGE_PAGE_SIZE));
}

#else

/* Memory allocation functions that use Intel C compiler intrinsics */

void *bench_malloc(size_t size)
{
	void *ptr = _mm_malloc(size, 4096);
	if (ptr != NULL)
		return ptr;

	printf("Error: Could not allocate memory (%zu Mbytes)\n",
			size / (1024 * 1024));
	exit(1);
}

void bench_free(void *ptr, size_t size)
{
	_mm_free(ptr);
}

#endif

void fill_matrix(fptype_t *A, size_t nelems)
{
	int j;
	for (j = 0; j < nelems; j++) {
#if defined(SINGLE_PREC) || defined(DOUBLE_PREC)
		A[j] = ((double)rand()) * 2.0 / RAND_MAX - 1.0;
#else
		A[j].real = ((double)rand()) * 2.0 / RAND_MAX - 1.0;
		A[j].imag = ((double)rand()) * 2.0 / RAND_MAX - 1.0;
#endif
	}
}
