/*******************************************************************************
!   Copyright(C) 2011-2012 Intel Corporation. All Rights Reserved.
!
!   The source code, information  and  material ("Material") contained herein is
!   owned  by Intel Corporation or its suppliers or licensors, and title to such
!   Material remains  with Intel Corporation  or its suppliers or licensors. The
!   Material  contains proprietary information  of  Intel or  its  suppliers and
!   licensors. The  Material is protected by worldwide copyright laws and treaty
!   provisions. No  part  of  the  Material  may  be  used,  copied, reproduced,
!   modified, published, uploaded, posted, transmitted, distributed or disclosed
!   in any way  without Intel's  prior  express written  permission. No  license
!   under  any patent, copyright  or  other intellectual property rights  in the
!   Material  is  granted  to  or  conferred  upon  you,  either  expressly,  by
!   implication, inducement,  estoppel or  otherwise.  Any  license  under  such
!   intellectual  property  rights must  be express  and  approved  by  Intel in
!   writing.
!
!   *Third Party trademarks are the property of their respective owners.
!
!   Unless otherwise  agreed  by Intel  in writing, you may not remove  or alter
!   this  notice or  any other notice embedded  in Materials by Intel or Intel's
!   suppliers or licensors in any way.
!
!*******************************************************************************
!   Content:
!       Utility functions for SGEMM, DGEMM, CGEMM, ZGEMM benchmarks
!******************************************************************************/

#ifndef _UTILS_H_
#define _UTILS_H_

/* Min and max macros */
#define MIN(a,b) ((a) < (b) ? (a) : (b))
#define MAX(a,b) ((a) > (b) ? (a) : (b))

/* A macro to 'fix' matrix leading dimension to avoid 4K aliasing */
#define FIX_LD_SHFT 128
#define FIX_LD_MULT 256
#define FIX_LD(x) ((x) % FIX_LD_MULT ? (x) : (x) + FIX_LD_SHFT)

/* Decorations for offload data and functions prototypes */
#if defined( __INTEL_OFFLOAD) && ! defined (_WINHOST)
#define TARGET_MIC_ATTR __attribute__((target(mic)))
#define TARGET_MIC_PUSH _Pragma("offload_attribute(push,target(mic))")
#define TARGET_MIC_POP _Pragma("offload_attribute(pop)")

#elif defined(__INTEL_OFFLOAD) && defined (_WINHOST)
#define TARGET_MIC_ATTR __declspec(target(mic))
//for push use inline #pragma offload_attribute(push,target(mic))
//for pop use inline #pragma offload_attribute(pop)

#else
#define TARGET_MIC_ATTR
#define TARGET_MIC_PUSH
#define TARGET_MIC_POP
#endif

#ifndef _WINHOST
TARGET_MIC_PUSH
#else
#pragma offload_attribute(push,target(mic))
#endif
#include "mkl.h"
#ifndef _WINHOST
TARGET_MIC_POP
#else
#pragma offload_attribute(pop)
#endif

/* Precision-dependent types */
#if defined(SINGLE_PREC)
#define fptype_t float
#elif defined (DOUBLE_PREC)
#define fptype_t double
#elif defined(COMPLEX)
#define fptype_t MKL_Complex8
#else
#define fptype_t MKL_Complex16
#endif

/* Memory allocation */
void *bench_malloc(size_t size);
void bench_free(void *ptr, size_t size);

/* Fill matrix with random values */
void fill_matrix(fptype_t *A, size_t nelems);

#endif
