#  Copyright 2012, 2013, 2014 Intel Corporation All Rights Reserved.
#
# This software is supplied under the terms of a license
# agreement or nondisclosure agreement with Intel Corp.
# and may not be copied or disclosed except in accordance
# with the terms of that agreement.
#
#  Author:  Christopher M. Cantalupo

import platform


class Factory(object):
    def __init__(self):
        self._classMap = {}

    def register(self, theName, theClass):
        self._classMap[theName] = theClass

    def class_names(self):
        return sorted(self._classMap.keys())

    def create(self, className):
        try:
            resultClass = self._classMap[className]
        except KeyError:
            nameList = '\n    '.join(self.class_names())
            errorString = 'No registered class named "{0}"\nAvailable classes:\n    {1}'.format(className, nameList)
            raise FactoryLookupError(errorString)
        return resultClass()

class FactoryLookupError(Exception):
    pass

def star_border(name):
    lineWidth = 80
    minStars = 5
    if len(name) < lineWidth - 2*minStars:
        leftStars = (lineWidth - len(name))/2
        rightStars = lineWidth - (leftStars + len(name))
        return '*'*leftStars + name + '*'*rightStars
    else:
        return '*'*minStars + name + '*'*minStars

def is_platform_windows():
    return platform.platform().lower().startswith('windows')

class NoSharedObjectError(Exception):
    def __str__(self):
       if is_platform_windows():
           errString = '\n'\
                       '    The Composer_XE_2013 redistributable package located here:\n'\
                       '    http://software.intel.com/en-us/articles/redistributable-libraries-for-intel-c-and-visual-fortran-composer-xe-2013-sp1-for-windows\n'\
                       '    can be installed to access the required shared object libraries.\n'
       else:
           errString = '\n'\
                       '    Make sure that compilervars.sh from Composer_XE is sourced in\n'\
                       '    your working environment.  The Composer_XE_2013 redistributable\n'\
                       '    package located here:\n'\
                       '    http://software.intel.com/en-us/articles/redistributable-libraries-for-the-intel-c-and-fortran-composer-xe-2013-sp1-for-linux\n'\
                       '    can be installed to access the required shared object libraries.\n'
       return errString
