#  Copyright 2012, 2013, 2014 Intel Corporation All Rights Reserved.
#
# This software is supplied under the terms of a license
# agreement or nondisclosure agreement with Intel Corp.
# and may not be copied or disclosed except in accordance
# with the terms of that agreement.
#
#  Author:  Christopher M. Cantalupo

import os
import re

import micp.kernel as micp_kernel
import micp.info as micp_info

class stream(micp_kernel.Kernel):
    def __init__(self):
        self.name = 'stream'
        self._paramNames = ['omp_num_threads']
        self._paramDefaults = {'omp_num_threads':'57'}

        self._categoryParams = {}

        self._categoryParams['test'] = [' ']

        maxCount = micp_info.Info().num_cores()
        threadsPerCore = 1
        coreConfig = range(1,maxCount+1)
        self._categoryParams['scaling'] = \
            ['--omp_num_threads {0}'.format(threadsPerCore*cc) for cc in coreConfig]

        optimalThreads = maxCount
        self._categoryParams['optimal'] = self._categoryParams['scaling'][-10:]
        self._categoryParams['optimal_quick'] = ['--omp_num_threads {0}'.format(optimalThreads)]
        self._categoryParams['scaling_quick'] = list(self._categoryParams['scaling'])
        self._categoryParams['scaling_core'] = list(self._categoryParams['scaling'])
        self._set_defaults_to_optimal()

    def path_aux_data(self, offType):
        result = []
        if offType == 'native':
            result.append(self.mic_library_find('libiomp5.so'))
        return result

    def path_dev_exec(self, offType):
        if offType == 'native':
            execName = 'stream_mic'
            return self._path_exec('k1om', execName)
        return None

    def path_host_exec(self, offType):
        return None

    def _do_unit_test(self):
        return True

    def offload_methods(self):
        return ['native']

    def param_type(self):
        return 'value'

    def independent_var(self, category):
        return 'omp_num_threads'

    def param_for_env(self):
        return ['omp_num_threads']

    def environment_dev(self):
        return {'LD_LIBRARY_PATH':'/tmp',
                'KMP_AFFINITY':'scatter'}

    def parse_desc(self, raw):
        lines = raw.splitlines()
        # Check validation
        failRE = re.compile('fail', re.IGNORECASE)
        if failRE.search(raw):
            eMessage = [line for line in lines
                        if failRE.search(line) or
                        line.strip().startswith("Expected") or
                        line.strip().startswith("Observed")]
            eMessage.insert(0, '')
            eMessage = '\n'.join(eMessage)
            raise micp_kernel.SelfCheckError(eMessage)
        # Get stream version information
        name = [line.strip() for line in lines
                if line.startswith('STREAM version')][0]
        # Get the number of threads
        numThreads = [line.split()[5] for line in lines
                      if line.startswith('Number of Threads requested = ')][0]
        result = '{0} with {1} threads'.format(name, numThreads)
        return result

    def parse_perf(self, raw):
        rate = float([line.split()[1] for line in raw.splitlines() if line.startswith('Triad: ')][-1])
        rate = str(rate/1000) # MB/s -> GB/s
        result = {}
        result['Task.Bandwidth'] = {'value':rate, 'units':'GB/s', 'rollup':True}
        return result
